/*
	*
	* Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* at your option, any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <desqui/DesQUI.hpp>

class DesQActionBarTitle;
class DesQActionBarItem;

class DesQActionBar : public QWidget {
	Q_OBJECT;

	public :
		DesQActionBar( QWidget *, Qt::Edge edge = Qt::BottomEdge );

		// An action can be uniquiely identified with (i, j) pair: j-th item on i-th page
		// Since the user already knows the page, we simply return the position of the item
		int addAction( int page, QString name, QIcon icon, QString target, QString tooltip, bool checkable, bool checked );

		void setIcon( int page, int item, QIcon icon );

		/* Enable/disable an item */
		void setItemEnabled( int page, int item );
		void setItemDisabled( int page, int item );

		void addSingleSpace( int page );
		void addStretch( int page );

		void addSeparator( int page );

		int pageCount() {

			return layoutList.count();
		};

	public Q_SLOTS:
		void switchToPage( int page );

	private:
		void createUI();

		DesQActionBarTitle *title;

		QStackedWidget *stack;
		QList<QHBoxLayout *> layoutList;

		/* page, List of items on that page */
		QMap<int, QList<DesQActionBarItem*> > itemList;

		bool mCompact = false;
		Qt::Edge mEdge;

	Q_SIGNALS:
		void action( QString );
};

class DesQActionBarItem : public QWidget {
	Q_OBJECT;

	public:
		DesQActionBarItem( QString name, QIcon icon, QString target, QString tooltip, bool checkable, bool checked, QWidget *parent );

		void setEnabled();
		void setDisabled();

		void setIcon( QIcon );

	private:
		QString mName;
		QString mTarget;

		QIcon mIcon;

		bool mPressed;
		bool mHover;

		bool mCheckable;
		bool mChecked;

		bool compact = false;

	protected:
		void enterEvent( QEvent* );
		void leaveEvent( QEvent* );

		void mouseReleaseEvent( QMouseEvent *mrEvent );
		void mousePressEvent( QMouseEvent *mpEvent );

		void paintEvent( QPaintEvent *pEvent );

	Q_SIGNALS:
		void clicked();
		void clicked( QString );

		void switchToPage( int );
};

class DesQActionBarSpace : public QWidget {
	Q_OBJECT;

	public:
		DesQActionBarSpace( QWidget *parent );
};

class DesQActionBarSeparator : public QWidget {
	Q_OBJECT;

	public:
		DesQActionBarSeparator( Qt::Edge edge, QWidget *parent );

	protected:
		void paintEvent( QPaintEvent *pEvent );
};
